// Copyright Marcus Del Favero 2025
// Licensed under the GNU AGPLv3 with an exception, see `README.md` for details
#[cfg(feature = "client")] mod renderer;
#[cfg(feature = "client")] mod local;

#[cfg(feature = "client")] pub use renderer::{Renderer as ForcefieldRenderer, RenderedForcefield};
#[cfg(feature = "client")] pub use local::LocalForcefield;

use glam::Vec2;

use super::{PlayerId, TeamId};

pub struct Forcefield {
	pub player_id: PlayerId,
	pub team_id: Option<TeamId>,
	pub pos: Vec2,
	pub strength: f32,
}

impl Forcefield {
	const RADIUS: f32 = 4.0;

	pub fn strength(pos_a: Vec2, pos_b: Vec2) -> Option<f32> {
		Forcefield::inv_strength(pos_a, pos_b).map(|inv| 1.0 - inv)
	}

	pub fn inv_strength(pos_a: Vec2, pos_b: Vec2) -> Option<f32> {
		let dist2 = pos_a.distance_squared(pos_b);
		(dist2 < Forcefield::RADIUS * Forcefield::RADIUS).then(|| dist2.sqrt() / Forcefield::RADIUS)
	}
}
